Imports System 
Imports System.Text 
Imports System.Data
Imports System.Data.SqlClient
Imports System.Data.Common
Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Configuration 
Imports System.Xml 
Imports System.Xml.Serialization
Imports SubSonic 
Imports SubSonic.Utilities
Namespace InfoPool.Data
    ''' <summary>
    ''' Strongly-typed collection for the Player class.
    ''' </summary>
    <Serializable()> _
    Partial Public Class PlayerCollection
        Inherits ActiveList(Of Player, PlayerCollection)
        Public Sub New()
        End Sub
    End Class
    ''' <summary>
    ''' This is an ActiveRecord class which wraps the Players table.
    ''' </summary>
    <Serializable()> _
    Partial Public Class Player
        Inherits ActiveRecord(Of Player)
#Region ".ctors and Default Settings"

        Public Sub New()
            SetSQLProps()
            InitSetDefaults()
            MarkNew()
        End Sub

        Public Sub New(ByVal useDatabaseDefaults As Boolean)
            SetSQLProps()
            If useDatabaseDefaults = True Then
                ForceDefaults()
            End If
            MarkNew()
        End Sub
        Public Sub New(ByVal keyID As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByKey(keyID)
        End Sub
        Public Sub New(ByVal columnName As String, ByVal columnValue As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByParam(columnName, columnValue)
        End Sub
        Private Sub InitSetDefaults()
            SetDefaults()
        End Sub

        Protected Shared Sub SetSQLProps()
            GetTableSchema()
        End Sub
#End Region

#Region "Schema and Query Accessor"

        Public Shared ReadOnly Property Schema() As TableSchema.Table
            Get
                If BaseSchema Is Nothing Then
                    SetSQLProps()
                End If
                Return BaseSchema
            End Get
        End Property
        Private Shared Sub GetTableSchema()
            If (Not IsSchemaInitialized) Then
                'Schema declaration
                Dim schema As TableSchema.Table = New TableSchema.Table("Players", TableType.Table, DataService.GetInstance("InfoPoolData"))
                schema.Columns = New TableSchema.TableColumnCollection()
                schema.SchemaName = "dbo"
                'columns


                Dim colvarId As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarId.ColumnName = "id"
                colvarId.DataType = DbType.Int32
                colvarId.MaxLength = 0
                colvarId.AutoIncrement = True
                colvarId.IsNullable = False
                colvarId.IsPrimaryKey = True
                colvarId.IsForeignKey = False
                colvarId.IsReadOnly = False


                schema.Columns.Add(colvarId)

                Dim colvarMemKey As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarMemKey.ColumnName = "MemKey"
                colvarMemKey.DataType = DbType.Guid
                colvarMemKey.MaxLength = 0
                colvarMemKey.AutoIncrement = False
                colvarMemKey.IsNullable = False
                colvarMemKey.IsPrimaryKey = False
                colvarMemKey.IsForeignKey = False
                colvarMemKey.IsReadOnly = False


                schema.Columns.Add(colvarMemKey)

                BaseSchema = schema

                'add this schema to the provider
                'so we can query it later
                DataService.Providers("InfoPoolData").AddSchema("Players", schema)
            End If
        End Sub
        Public Shared Function CreateQuery() As Query
            Return New Query(Schema)
        End Function

#End Region

#Region "Props"


        <XmlAttribute("Id")> _
        Public Property Id() As Integer
            Get
                Return GetColumnValue(Of Integer)("id")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("id", Value)
            End Set
        End Property



        <XmlAttribute("MemKey")> _
        Public Property MemKey() As Guid
            Get
                Return GetColumnValue(Of Guid)("MemKey")
            End Get

            Set(ByVal value As Guid)
                SetColumnValue("MemKey", Value)
            End Set
        End Property




#End Region




#Region "PrimaryKey Methods"

        Public Function PlayersInStats() As InfoPool.Data.PlayersInStatCollection

            Return New InfoPool.Data.PlayersInStatCollection().Where(PlayersInStat.Columns.PlayerID, Id).Load()

        End Function

        Public Function PlayersInTeams() As InfoPool.Data.PlayersInTeamCollection

            Return New InfoPool.Data.PlayersInTeamCollection().Where(PlayersInTeam.Columns.PlayerID, Id).Load()

        End Function

#End Region







        'no foreign key tables defined (0)



        'no ManyToMany tables defined (0)


#Region "ObjectDataSource support"

        ''' <summary>
        ''' Inserts a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Insert(ByVal varMemKey As Guid)
            Dim item As Player = New Player()

            item.MemKey = varMemKey

            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
        ''' <summary>
        ''' Updates a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Update(ByVal varId As Integer, ByVal varMemKey As Guid)
            Dim item As Player = New Player()

            item.Id = varId

            item.MemKey = varMemKey

            item.IsNew = False
            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
#End Region
#Region "Columns Struct"
        Public Structure Columns
            Dim x As Integer

            Public Shared Id As String = "id"

            Public Shared MemKey As String = "MemKey"

        End Structure
#End Region
    End Class
End Namespace
