Imports System 
Imports System.Text 
Imports System.Data
Imports System.Data.SqlClient
Imports System.Data.Common
Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Configuration 
Imports System.Xml 
Imports System.Xml.Serialization
Imports SubSonic 
Imports SubSonic.Utilities
Namespace InfoPool.Data
    ''' <summary>
    ''' Strongly-typed collection for the PlayersInTeam class.
    ''' </summary>
    <Serializable()> _
    Partial Public Class PlayersInTeamCollection
        Inherits ActiveList(Of PlayersInTeam, PlayersInTeamCollection)
        Public Sub New()
        End Sub
    End Class
    ''' <summary>
    ''' This is an ActiveRecord class which wraps the PlayersInTeams table.
    ''' </summary>
    <Serializable()> _
    Partial Public Class PlayersInTeam
        Inherits ActiveRecord(Of PlayersInTeam)
#Region ".ctors and Default Settings"

        Public Sub New()
            SetSQLProps()
            InitSetDefaults()
            MarkNew()
        End Sub

        Public Sub New(ByVal useDatabaseDefaults As Boolean)
            SetSQLProps()
            If useDatabaseDefaults = True Then
                ForceDefaults()
            End If
            MarkNew()
        End Sub
        Public Sub New(ByVal keyID As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByKey(keyID)
        End Sub
        Public Sub New(ByVal columnName As String, ByVal columnValue As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByParam(columnName, columnValue)
        End Sub
        Private Sub InitSetDefaults()
            SetDefaults()
        End Sub

        Protected Shared Sub SetSQLProps()
            GetTableSchema()
        End Sub
#End Region

#Region "Schema and Query Accessor"

        Public Shared ReadOnly Property Schema() As TableSchema.Table
            Get
                If BaseSchema Is Nothing Then
                    SetSQLProps()
                End If
                Return BaseSchema
            End Get
        End Property
        Private Shared Sub GetTableSchema()
            If (Not IsSchemaInitialized) Then
                'Schema declaration
                Dim schema As TableSchema.Table = New TableSchema.Table("PlayersInTeams", TableType.Table, DataService.GetInstance("InfoPoolData"))
                schema.Columns = New TableSchema.TableColumnCollection()
                schema.SchemaName = "dbo"
                'columns


                Dim colvarId As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarId.ColumnName = "ID"
                colvarId.DataType = DbType.Int32
                colvarId.MaxLength = 0
                colvarId.AutoIncrement = True
                colvarId.IsNullable = False
                colvarId.IsPrimaryKey = True
                colvarId.IsForeignKey = False
                colvarId.IsReadOnly = False


                schema.Columns.Add(colvarId)

                Dim colvarPlayerID As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarPlayerID.ColumnName = "PlayerID"
                colvarPlayerID.DataType = DbType.Int32
                colvarPlayerID.MaxLength = 0
                colvarPlayerID.AutoIncrement = False
                colvarPlayerID.IsNullable = False
                colvarPlayerID.IsPrimaryKey = False
                colvarPlayerID.IsForeignKey = True
                colvarPlayerID.IsReadOnly = False


                colvarPlayerID.ForeignKeyTableName = "Players"

                schema.Columns.Add(colvarPlayerID)

                Dim colvarTeamID As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarTeamID.ColumnName = "TeamID"
                colvarTeamID.DataType = DbType.Int32
                colvarTeamID.MaxLength = 0
                colvarTeamID.AutoIncrement = False
                colvarTeamID.IsNullable = False
                colvarTeamID.IsPrimaryKey = False
                colvarTeamID.IsForeignKey = True
                colvarTeamID.IsReadOnly = False


                colvarTeamID.ForeignKeyTableName = "Teams"

                schema.Columns.Add(colvarTeamID)

                BaseSchema = schema

                'add this schema to the provider
                'so we can query it later
                DataService.Providers("InfoPoolData").AddSchema("PlayersInTeams", schema)
            End If
        End Sub
        Public Shared Function CreateQuery() As Query
            Return New Query(Schema)
        End Function

#End Region

#Region "Props"


        <XmlAttribute("Id")> _
        Public Property Id() As Integer
            Get
                Return GetColumnValue(Of Integer)("ID")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("ID", Value)
            End Set
        End Property



        <XmlAttribute("PlayerID")> _
        Public Property PlayerID() As Integer
            Get
                Return GetColumnValue(Of Integer)("PlayerID")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("PlayerID", Value)
            End Set
        End Property



        <XmlAttribute("TeamID")> _
        Public Property TeamID() As Integer
            Get
                Return GetColumnValue(Of Integer)("TeamID")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("TeamID", Value)
            End Set
        End Property




#End Region










#Region "ForeignKey Methods"

        ''' <summary>
        ''' Returns a Player ActiveRecord object related to this PlayersInTeam
        ''' </summary>
        Public Property Player() As InfoPool.Data.Player
            Get
                Return InfoPool.Data.Player.FetchByID(Me.PlayerID)
            End Get
            Set(ByVal value As InfoPool.Data.Player)
                SetColumnValue("PlayerID", Value.Id)
            End Set
        End Property

        ''' <summary>
        ''' Returns a Team ActiveRecord object related to this PlayersInTeam
        ''' </summary>
        Public Property Team() As InfoPool.Data.Team
            Get
                Return InfoPool.Data.Team.FetchByID(Me.TeamID)
            End Get
            Set(ByVal value As InfoPool.Data.Team)
                SetColumnValue("TeamID", Value.Id)
            End Set
        End Property

#End Region



        'no ManyToMany tables defined (0)


#Region "ObjectDataSource support"

        ''' <summary>
        ''' Inserts a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Insert(ByVal varPlayerID As Integer, ByVal varTeamID As Integer)
            Dim item As PlayersInTeam = New PlayersInTeam()

            item.PlayerID = varPlayerID

            item.TeamID = varTeamID

            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
        ''' <summary>
        ''' Updates a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Update(ByVal varId As Integer, ByVal varPlayerID As Integer, ByVal varTeamID As Integer)
            Dim item As PlayersInTeam = New PlayersInTeam()

            item.Id = varId

            item.PlayerID = varPlayerID

            item.TeamID = varTeamID

            item.IsNew = False
            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
#End Region
#Region "Columns Struct"
        Public Structure Columns
            Dim x As Integer

            Public Shared Id As String = "ID"

            Public Shared PlayerID As String = "PlayerID"

            Public Shared TeamID As String = "TeamID"

        End Structure
#End Region
    End Class
End Namespace
