Imports System 
Imports System.Text 
Imports System.Data
Imports System.Data.SqlClient
Imports System.Data.Common
Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Configuration 
Imports System.Xml 
Imports System.Xml.Serialization
Imports SubSonic 
Imports SubSonic.Utilities
Namespace InfoPool.Data
    ''' <summary>
    ''' Strongly-typed collection for the Seasons class.
    ''' </summary>
    <Serializable()> _
    Partial Public Class SeasonsCollection
        Inherits ActiveList(Of Seasons, SeasonsCollection)
        Public Sub New()
        End Sub
    End Class
    ''' <summary>
    ''' This is an ActiveRecord class which wraps the Seasons table.
    ''' </summary>
    <Serializable()> _
    Partial Public Class Seasons
        Inherits ActiveRecord(Of Seasons)
#Region ".ctors and Default Settings"

        Public Sub New()
            SetSQLProps()
            InitSetDefaults()
            MarkNew()
        End Sub

        Public Sub New(ByVal useDatabaseDefaults As Boolean)
            SetSQLProps()
            If useDatabaseDefaults = True Then
                ForceDefaults()
            End If
            MarkNew()
        End Sub
        Public Sub New(ByVal keyID As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByKey(keyID)
        End Sub
        Public Sub New(ByVal columnName As String, ByVal columnValue As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByParam(columnName, columnValue)
        End Sub
        Private Sub InitSetDefaults()
            SetDefaults()
        End Sub

        Protected Shared Sub SetSQLProps()
            GetTableSchema()
        End Sub
#End Region

#Region "Schema and Query Accessor"

        Public Shared ReadOnly Property Schema() As TableSchema.Table
            Get
                If BaseSchema Is Nothing Then
                    SetSQLProps()
                End If
                Return BaseSchema
            End Get
        End Property
        Private Shared Sub GetTableSchema()
            If (Not IsSchemaInitialized) Then
                'Schema declaration
                Dim schema As TableSchema.Table = New TableSchema.Table("Seasons", TableType.Table, DataService.GetInstance("InfoPoolData"))
                schema.Columns = New TableSchema.TableColumnCollection()
                schema.SchemaName = "dbo"
                'columns


                Dim colvarId As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarId.ColumnName = "ID"
                colvarId.DataType = DbType.Int32
                colvarId.MaxLength = 0
                colvarId.AutoIncrement = True
                colvarId.IsNullable = False
                colvarId.IsPrimaryKey = True
                colvarId.IsForeignKey = False
                colvarId.IsReadOnly = False


                schema.Columns.Add(colvarId)

                Dim colvarStartDate As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarStartDate.ColumnName = "StartDate"
                colvarStartDate.DataType = DbType.DateTime
                colvarStartDate.MaxLength = 0
                colvarStartDate.AutoIncrement = False
                colvarStartDate.IsNullable = False
                colvarStartDate.IsPrimaryKey = False
                colvarStartDate.IsForeignKey = False
                colvarStartDate.IsReadOnly = False


                schema.Columns.Add(colvarStartDate)

                Dim colvarEndDate As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarEndDate.ColumnName = "EndDate"
                colvarEndDate.DataType = DbType.DateTime
                colvarEndDate.MaxLength = 0
                colvarEndDate.AutoIncrement = False
                colvarEndDate.IsNullable = False
                colvarEndDate.IsPrimaryKey = False
                colvarEndDate.IsForeignKey = False
                colvarEndDate.IsReadOnly = False


                schema.Columns.Add(colvarEndDate)

                Dim colvarSimpleName As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarSimpleName.ColumnName = "SimpleName"
                colvarSimpleName.DataType = DbType.String
                colvarSimpleName.MaxLength = 50
                colvarSimpleName.AutoIncrement = False
                colvarSimpleName.IsNullable = False
                colvarSimpleName.IsPrimaryKey = False
                colvarSimpleName.IsForeignKey = False
                colvarSimpleName.IsReadOnly = False


                schema.Columns.Add(colvarSimpleName)

                BaseSchema = schema

                'add this schema to the provider
                'so we can query it later
                DataService.Providers("InfoPoolData").AddSchema("Seasons", schema)
            End If
        End Sub
        Public Shared Function CreateQuery() As Query
            Return New Query(Schema)
        End Function

#End Region

#Region "Props"


        <XmlAttribute("Id")> _
        Public Property Id() As Integer
            Get
                Return GetColumnValue(Of Integer)("ID")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("ID", Value)
            End Set
        End Property



        <XmlAttribute("StartDate")> _
        Public Property StartDate() As DateTime
            Get
                Return GetColumnValue(Of DateTime)("StartDate")
            End Get

            Set(ByVal value As DateTime)
                SetColumnValue("StartDate", Value)
            End Set
        End Property



        <XmlAttribute("EndDate")> _
        Public Property EndDate() As DateTime
            Get
                Return GetColumnValue(Of DateTime)("EndDate")
            End Get

            Set(ByVal value As DateTime)
                SetColumnValue("EndDate", Value)
            End Set
        End Property



        <XmlAttribute("SimpleName")> _
        Public Property SimpleName() As String
            Get
                Return GetColumnValue(Of String)("SimpleName")
            End Get

            Set(ByVal value As String)
                SetColumnValue("SimpleName", Value)
            End Set
        End Property




#End Region




#Region "PrimaryKey Methods"

        Public Function TeamsRecords() As InfoPool.Data.TeamsCollection

            Return New InfoPool.Data.TeamsCollection().Where(Teams.Columns.SeasonID, Id).Load()

        End Function

#End Region







        'no foreign key tables defined (0)



        'no ManyToMany tables defined (0)


#Region "ObjectDataSource support"

        ''' <summary>
        ''' Inserts a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Insert(ByVal varStartDate As DateTime, ByVal varEndDate As DateTime, ByVal varSimpleName As String)
            Dim item As Seasons = New Seasons()

            item.StartDate = varStartDate

            item.EndDate = varEndDate

            item.SimpleName = varSimpleName

            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
        ''' <summary>
        ''' Updates a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Update(ByVal varId As Integer, ByVal varStartDate As DateTime, ByVal varEndDate As DateTime, ByVal varSimpleName As String)
            Dim item As Seasons = New Seasons()

            item.Id = varId

            item.StartDate = varStartDate

            item.EndDate = varEndDate

            item.SimpleName = varSimpleName

            item.IsNew = False
            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
#End Region
#Region "Columns Struct"
        Public Structure Columns
            Dim x As Integer

            Public Shared Id As String = "ID"

            Public Shared StartDate As String = "StartDate"

            Public Shared EndDate As String = "EndDate"

            Public Shared SimpleName As String = "SimpleName"

        End Structure
#End Region
    End Class
End Namespace
