Imports System 
Imports System.Text 
Imports System.Data
Imports System.Data.SqlClient
Imports System.Data.Common
Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Configuration 
Imports System.Xml 
Imports System.Xml.Serialization
Imports SubSonic 
Imports SubSonic.Utilities
Namespace InfoPool.Data
    ''' <summary>
    ''' Strongly-typed collection for the Team class.
    ''' </summary>
    <Serializable()> _
    Partial Public Class TeamCollection
        Inherits ActiveList(Of Team, TeamCollection)
        Public Sub New()
        End Sub
    End Class
    ''' <summary>
    ''' This is an ActiveRecord class which wraps the Teams table.
    ''' </summary>
    <Serializable()> _
    Partial Public Class Team
        Inherits ActiveRecord(Of Team)
#Region ".ctors and Default Settings"

        Public Sub New()
            SetSQLProps()
            InitSetDefaults()
            MarkNew()
        End Sub

        Public Sub New(ByVal useDatabaseDefaults As Boolean)
            SetSQLProps()
            If useDatabaseDefaults = True Then
                ForceDefaults()
            End If
            MarkNew()
        End Sub
        Public Sub New(ByVal keyID As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByKey(keyID)
        End Sub
        Public Sub New(ByVal columnName As String, ByVal columnValue As Object)
            SetSQLProps()
            InitSetDefaults()
            LoadByParam(columnName, columnValue)
        End Sub
        Private Sub InitSetDefaults()
            SetDefaults()
        End Sub

        Protected Shared Sub SetSQLProps()
            GetTableSchema()
        End Sub
#End Region

#Region "Schema and Query Accessor"

        Public Shared ReadOnly Property Schema() As TableSchema.Table
            Get
                If BaseSchema Is Nothing Then
                    SetSQLProps()
                End If
                Return BaseSchema
            End Get
        End Property
        Private Shared Sub GetTableSchema()
            If (Not IsSchemaInitialized) Then
                'Schema declaration
                Dim schema As TableSchema.Table = New TableSchema.Table("Teams", TableType.Table, DataService.GetInstance("InfoPoolData"))
                schema.Columns = New TableSchema.TableColumnCollection()
                schema.SchemaName = "dbo"
                'columns


                Dim colvarId As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarId.ColumnName = "id"
                colvarId.DataType = DbType.Int32
                colvarId.MaxLength = 0
                colvarId.AutoIncrement = True
                colvarId.IsNullable = False
                colvarId.IsPrimaryKey = True
                colvarId.IsForeignKey = False
                colvarId.IsReadOnly = False


                schema.Columns.Add(colvarId)

                Dim colvarName As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarName.ColumnName = "Name"
                colvarName.DataType = DbType.String
                colvarName.MaxLength = 50
                colvarName.AutoIncrement = False
                colvarName.IsNullable = False
                colvarName.IsPrimaryKey = False
                colvarName.IsForeignKey = False
                colvarName.IsReadOnly = False


                schema.Columns.Add(colvarName)

                Dim colvarSeasonID As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarSeasonID.ColumnName = "SeasonID"
                colvarSeasonID.DataType = DbType.Int32
                colvarSeasonID.MaxLength = 0
                colvarSeasonID.AutoIncrement = False
                colvarSeasonID.IsNullable = False
                colvarSeasonID.IsPrimaryKey = False
                colvarSeasonID.IsForeignKey = True
                colvarSeasonID.IsReadOnly = False


                colvarSeasonID.ForeignKeyTableName = "Seasons"

                schema.Columns.Add(colvarSeasonID)

                Dim colvarLeaderMemId As TableSchema.TableColumn = New TableSchema.TableColumn(schema)
                colvarLeaderMemId.ColumnName = "LeaderMemId"
                colvarLeaderMemId.DataType = DbType.Guid
                colvarLeaderMemId.MaxLength = 0
                colvarLeaderMemId.AutoIncrement = False
                colvarLeaderMemId.IsNullable = False
                colvarLeaderMemId.IsPrimaryKey = False
                colvarLeaderMemId.IsForeignKey = False
                colvarLeaderMemId.IsReadOnly = False


                schema.Columns.Add(colvarLeaderMemId)

                BaseSchema = schema

                'add this schema to the provider
                'so we can query it later
                DataService.Providers("InfoPoolData").AddSchema("Teams", schema)
            End If
        End Sub
        Public Shared Function CreateQuery() As Query
            Return New Query(Schema)
        End Function

#End Region

#Region "Props"


        <XmlAttribute("Id")> _
        Public Property Id() As Integer
            Get
                Return GetColumnValue(Of Integer)("id")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("id", Value)
            End Set
        End Property



        <XmlAttribute("Name")> _
        Public Property Name() As String
            Get
                Return GetColumnValue(Of String)("Name")
            End Get

            Set(ByVal value As String)
                SetColumnValue("Name", Value)
            End Set
        End Property



        <XmlAttribute("SeasonID")> _
        Public Property SeasonID() As Integer
            Get
                Return GetColumnValue(Of Integer)("SeasonID")
            End Get

            Set(ByVal value As Integer)
                SetColumnValue("SeasonID", Value)
            End Set
        End Property



        <XmlAttribute("LeaderMemId")> _
        Public Property LeaderMemId() As Guid
            Get
                Return GetColumnValue(Of Guid)("LeaderMemId")
            End Get

            Set(ByVal value As Guid)
                SetColumnValue("LeaderMemId", Value)
            End Set
        End Property




#End Region




#Region "PrimaryKey Methods"

        Public Function Games() As InfoPool.Data.GameCollection

            Return New InfoPool.Data.GameCollection().Where(Game.Columns.Team1id, Id).Load()

        End Function

        Public Function GamesFromTeam() As InfoPool.Data.GameCollection

            Return New InfoPool.Data.GameCollection().Where(Game.Columns.Team2id, Id).Load()

        End Function

        Public Function PlayersInTeams() As InfoPool.Data.PlayersInTeamCollection

            Return New InfoPool.Data.PlayersInTeamCollection().Where(PlayersInTeam.Columns.TeamID, Id).Load()

        End Function

#End Region







#Region "ForeignKey Methods"

        ''' <summary>
        ''' Returns a Season ActiveRecord object related to this Team
        ''' </summary>
        Public Property Season() As InfoPool.Data.Season
            Get
                Return InfoPool.Data.Season.FetchByID(Me.SeasonID)
            End Get
            Set(ByVal value As InfoPool.Data.Season)
                SetColumnValue("SeasonID", Value.Id)
            End Set
        End Property

#End Region



        'no ManyToMany tables defined (0)


#Region "ObjectDataSource support"

        ''' <summary>
        ''' Inserts a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Insert(ByVal varName As String, ByVal varSeasonID As Integer, ByVal varLeaderMemId As Guid)
            Dim item As Team = New Team()

            item.Name = varName

            item.SeasonID = varSeasonID

            item.LeaderMemId = varLeaderMemId

            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
        ''' <summary>
        ''' Updates a record, can be used with the Object Data Source
        ''' </summary>
        Public Shared Sub Update(ByVal varId As Integer, ByVal varName As String, ByVal varSeasonID As Integer, ByVal varLeaderMemId As Guid)
            Dim item As Team = New Team()

            item.Id = varId

            item.Name = varName

            item.SeasonID = varSeasonID

            item.LeaderMemId = varLeaderMemId

            item.IsNew = False
            If Not System.Web.HttpContext.Current Is Nothing Then
                item.Save(System.Web.HttpContext.Current.User.Identity.Name)
            Else
                item.Save(System.Threading.Thread.CurrentPrincipal.Identity.Name)
            End If
        End Sub
#End Region
#Region "Columns Struct"
        Public Structure Columns
            Dim x As Integer

            Public Shared Id As String = "id"

            Public Shared Name As String = "Name"

            Public Shared SeasonID As String = "SeasonID"

            Public Shared LeaderMemId As String = "LeaderMemId"

        End Structure
#End Region
    End Class
End Namespace
